#!/home/cmwilson/bin/sun/scsh \
-o big-scheme -e main -s
!#

;;; convert-scm

(define *include-dir*   "/home/cmwilson/html/meta/includes")
(define *excluded-dirs* '("/home/cmwilson/html/meta"))
(define *verbose* #f)


;;; Is file a newer than file b?
;;; If one doesn't exist, it is deemed older.

(define newer?
  (lambda (file-a file-b)
    (cond ((file-not-exists? file-a) #f)
	  ((file-not-exists? file-b) #t)
	  (else (> (file-last-mod file-a) (file-last-mod file-b))))))


;;; return a list of source-files in dir
;;; (available to .setup scripts)

(define source-files
  (lambda (source-dir . patterns)
    (with-cwd source-dir
      (apply glob patterns))))


;;; Verify the proper existance of a symlink
;;; (available to .setup scripts)

(define verify-symlink
  (lambda (source-dir)
    (lambda (name)
      (let ((source (resolve-file-name name source-dir)))
	;(format #t "verify-symlink: ~a -> ~a~%" name source)
	(unless (and (file-exists? name)
		     (file-symlink? name)
		     (string=? source (read-symlink name)))
		(format #t "symlink for ~a broken; recreating~%" name)
		(create-symlink source name #t))))))


;;; Look for a file named ./.setup
;;; If it's there, it's a script that updates data in this directory; load it.

(define run-setup
  (let ((setup-file ".setup.scm"))
    (lambda ()
      (when (file-exists? setup-file)
	    (when *verbose* (format #t "Running .setup in ~a~%" (cwd)))
	    (load setup-file)
	    (when *verbose* (format #t "Finished .setup in ~a~%" (cwd)))
	    ))))


;;; Is this a dangling symlink?

(define dangling-symlink?
  (lambda (file)
    (and (file-exists? file #f)
	 (file-symlink? file)
	 (file-not-exists? file #t))))


(define convert
  (lambda (file source-dir dest-dir)
    (cond ((dangling-symlink? file)
	   (format #t "dangling symlink in convert: ~s~%" file)
	   #f)
	  ((file-directory? file)
	   (let ((new-source (resolve-file-name file source-dir))
		 (new-dest   (resolve-file-name file dest-dir)))
	     (unless (member new-source *excluded-dirs*)  ; foo/./bar breaks this
		     (when *verbose* (format #t "Converting directory: ~a~%" new-source))
		     (when (file-not-exists? new-dest)
			   (format #t "Creating directory: ~a~%" new-dest)
			   (create-directory new-dest))
		     (with-cwd file
		       (run-setup)
		       (for-each (lambda (f)
				   (convert f new-source new-dest))
				 (directory-files))))))
	  ((file-regular? file)
	   (bind ((dir name ext (parse-file-name file)))
  	     (let* ((new-extension (if (member ext '(".schemed-html" ".pre" ".prehtml"))
				       ".html"
				       ext))
		    (dest-file (resolve-file-name (string-append name new-extension) dest-dir)))
	       (if (newer? file dest-file)
		   (begin
		     (format #t "Updating ~a~%" file)
		     (let ((file-text (call-with-input-file file read-all)))
		       (when (file-exists? dest-file)
			     (delete-file dest-file))
		       (with-output-to-file dest-file
			 (lambda ()
			   (case-equal ext
			     ((".pre")          (display (plain-file->html name file-text)))
			     ((".schemed-html") (tree-for-each display
							       (schemed-html->html file-text)))
			     ((".prehtml")      (display (prehtml->html file-text)))
			     ((".cgi")          (display file-text)
			      (force-output (current-output-port))
			      (ensure-executable dest-file))
			     (else              (display file-text)))))))
		   (when *verbose* (format #t "Up to date: ~a~%" file))))))
	  (else (error 'convert "unrecognized filetype" file)))))


;;; do it

(define usage
  (lambda (name)
    (format #t (string-append "usage:~%"
			      "  ~a <html-src-directory> <html-dest-directory>~%"
			      "    moves everything in the src directory to the~%"
			      "    dest directory with the necessary conversions.~%"
			      " also:~%"
			      "  ~a --help~%"
			      "  ~a --verbose~%")
	    name name name)))

(define main
  (lambda (arg-list)
    (let ((prog-name (simplify-file-name (arg arg-list 1))))
      (let* ((set-verbose (lambda ()
			    (format #t "Verbose on~%")
			    (set! *verbose* #t)))
	     (exit-with-help (lambda ()
			       (usage prog-name)
			       (error "invalid arguments: ~a" arg-list)))
	     (args (let loop ((args (cdr arg-list)) (new-args '()))
		     (if (null? args)
			 (reverse new-args)
			 (let ((arg (car args)))
			   (case-equal arg
			     (("-h" "-?" "--help") (exit-with-help))
			     (("-v" "--verbose")   (set-verbose)
			                           (loop (cdr args) new-args))
			     (else (loop (cdr args) (cons arg new-args)))))))))
	(if (not (= (length args) 2))
	    (exit-with-help)
	    (let ((source-dir (directory-as-file-name (arg args 1)))
		  (dest-dir   (directory-as-file-name (arg args 2))))
	      (when *verbose*
		    (format #t (string-append "source-dir: ~a~%"
					      "dest-dir:   ~a~%")
			    source-dir dest-dir))
	      (cond ((file-not-exists? source-dir)
		     (error "source directory does not exist: ~a~%" source-dir))
		    ((file-not-exists? dest-dir)
		     (error "destination directory does not exist: ~a~%" dest-dir))
		    (else
		     (with-cwd source-dir
		       (convert "." source-dir dest-dir))))))))))


(define test (lambda () (main '("prog" "-v" "/home/cmwilson/html" "/home/cmwilson/public_html"))))
(define qtest (lambda () (main '("prog" "/home/cmwilson/html" "/home/cmwilson/public_html"))))
